@@ -11,100 +11,15 @@ import codecs
 import os
 import sys
 import re
-import shutil
+import pathlib
 
 import pytest
 
-from sphinx.application import Sphinx
 from sphinx.errors import ExtensionError
-from sphinx.util.docutils import docutils_namespace
-
-from sphinx_gallery.gen_rst import MixedEncodingStringIO
-from sphinx_gallery import sphinx_compatibility
 from sphinx_gallery.gen_gallery import (check_duplicate_filenames,
-                                        collect_gallery_files)
-from sphinx_gallery.utils import _TempDir
-
-
-@pytest.fixture
-def conf_file(request):
-    env = request.node.get_marker('conf_file')
-    kwargs = env.kwargs if env else {}
-    result = {
-        'content': "",
-    }
-    result.update(kwargs)
-
-    return result
-
-
-@pytest.fixture
-def tempdir():
-    """
-    temporary directory that wrapped with `path` class.
-    this fixture is for compat with old test implementation.
-    """
-    return _TempDir()
-
-
-class SphinxAppWrapper(object):
-    """Wrapper for sphinx.application.Application.
-
-    This allows to control when the sphinx application is initialized, since
-    part of the sphinx-gallery build is done in
-    sphinx.application.Application.__init__ and the remainder is done in
-    sphinx.application.Application.build.
-
-    """
-    def __init__(self, srcdir, confdir, outdir, doctreedir, buildername,
-                 **kwargs):
-        self.srcdir = srcdir
-        self.confdir = confdir
-        self.outdir = outdir
-        self.doctreedir = doctreedir
-        self.buildername = buildername
-        self.kwargs = kwargs
-
-    def create_sphinx_app(self):
-        # Avoid warnings about re-registration, see:
-        # https://github.com/sphinx-doc/sphinx/issues/5038
-        with docutils_namespace():
-            app = Sphinx(self.srcdir, self.confdir, self.outdir,
-                         self.doctreedir, self.buildername, **self.kwargs)
-        sphinx_compatibility._app = app
-        return app
-
-    def build_sphinx_app(self, *args, **kwargs):
-        app = self.create_sphinx_app()
-        app.build(*args, **kwargs)
-        return app
-
-
-@pytest.fixture
-def sphinx_app_wrapper(tempdir, conf_file):
-    _fixturedir = os.path.join(os.path.dirname(__file__), 'testconfs')
-    srcdir = os.path.join(tempdir, "config_test")
-    shutil.copytree(_fixturedir, srcdir)
-    shutil.copytree(os.path.join(_fixturedir, "src"),
-                    os.path.join(tempdir, "examples"))
-
-    base_config = """
-import os
-import sphinx_gallery
-extensions = ['sphinx_gallery.gen_gallery']
-exclude_patterns = ['_build']
-source_suffix = '.rst'
-master_doc = 'index'
-# General information about the project.
-project = u'Sphinx-Gallery <Tests>'\n\n
-"""
-    with open(os.path.join(srcdir, "conf.py"), "w") as conffile:
-        conffile.write(base_config + conf_file['content'])
-
-    return SphinxAppWrapper(
-        srcdir, srcdir, os.path.join(srcdir, "_build"),
-        os.path.join(srcdir, "_build", "toctree"),
-        "html", warning=MixedEncodingStringIO())
+                                        check_spaces_in_filenames,
+                                        collect_gallery_files,
+                                        write_computation_times)
 
 
 def test_default_config(sphinx_app_wrapper):
@@ -138,6 +53,34 @@ def test_no_warning_simple_config(sphinx_app_wrapper):
     assert build_warn == ''
 
 
+@pytest.mark.parametrize('err_class, err_match', [
+    pytest.param(ValueError, 'Unknown module resetter',
+                 id='Resetter unknown',
+                 marks=pytest.mark.conf_file(
+                     content="sphinx_gallery_conf={'reset_modules': ('f',)}")),
+    pytest.param(ValueError, 'Module resetter .* was not callab',
+                 id='Resetter not callable',
+                 marks=pytest.mark.conf_file(
+                     content="sphinx_gallery_conf={'reset_modules': (1.,),}")),
+])
+def test_bad_reset(sphinx_app_wrapper, err_class, err_match):
+    with pytest.raises(err_class, match=err_match):
+        sphinx_app_wrapper.create_sphinx_app()
+
+
+@pytest.mark.parametrize('err_class, err_match', [
+    pytest.param(ValueError, 'Unknown css', id='CSS str error',
+                 marks=pytest.mark.conf_file(
+                     content="sphinx_gallery_conf={'css': ('foo',)}")),
+    pytest.param(TypeError, 'must be list or tuple', id="CSS type error",
+                 marks=pytest.mark.conf_file(
+                     content="sphinx_gallery_conf={'css': 1.}")),
+])
+def test_bad_css(sphinx_app_wrapper, err_class, err_match):
+    with pytest.raises(err_class, match=err_match):
+        sphinx_app_wrapper.create_sphinx_app()
+
+
 @pytest.mark.conf_file(content="""
 sphinx_gallery_conf = {
     'mod_example_dir' : os.path.join('modules', 'gen'),
@@ -184,7 +127,7 @@ def test_duplicate_files_warn(sphinx_app_wrapper):
     sphinx_app = sphinx_app_wrapper.create_sphinx_app()
 
     files = ['./a/file1.py', './a/file2.py', 'a/file3.py', './b/file1.py']
-    msg = ("Duplicate file name(s) found. Having duplicate file names "
+    msg = ("Duplicate example file name(s) found. Having duplicate file names "
            "will break some links. List of files: {}")
     m = "['./b/file1.py']" if sys.version_info[0] >= 3 else "[u'./b/file1.py']"
 
@@ -199,6 +142,27 @@ def test_duplicate_files_warn(sphinx_app_wrapper):
     assert msg.format(m) in build_warn
 
 
+def test_spaces_in_files_warn(sphinx_app_wrapper):
+    """Test for a exception when an example filename has a space in it."""
+    sphinx_app = sphinx_app_wrapper.create_sphinx_app()
+
+    files = ['./a/file1.py', './a/file2.py', './a/file 3.py']
+    msg = ("Example file name(s) with space(s) found. Having space(s) in "
+           "file names will break some links. "
+           "List of files: {}")
+    m = "['./a/file 3.py']" if sys.version_info[0] >= 3 else "[u'./a/file 3.py']"
+
+    # No warning because no filename with space
+    check_spaces_in_filenames(files[:-1])
+    build_warn = sphinx_app._warning.getvalue()
+    assert build_warn == ''
+
+    # Warning because last file has space
+    check_spaces_in_filenames(files)
+    build_warn = sphinx_app._warning.getvalue()
+    assert msg.format(m) in build_warn
+
+
 def _check_order(sphinx_app, key):
     index_fname = os.path.join(sphinx_app.outdir, '..', 'ex', 'index.rst')
     order = list()
@@ -261,7 +225,7 @@ def test_example_sorting_title(sphinx_app_wrapper):
     _check_order(sphinx_app, 'title')
 
 
-def test_collect_gallery_files(sphinx_app_wrapper, tmpdir):
+def test_collect_gallery_files(tmpdir, gallery_conf):
     """Test that example files are collected properly."""
     rel_filepaths = ['examples/file1.py',
                      'examples/test.rst',
@@ -278,7 +242,7 @@ def test_collect_gallery_files(sphinx_app_wrapper, tmpdir):
 
     examples_path = tmpdir.join('examples')
     dirs = [examples_path.strpath]
-    collected_files = set(collect_gallery_files(dirs))
+    collected_files = set(collect_gallery_files(dirs, gallery_conf))
     expected_files = set(
         [ap.strpath for ap in abs_paths
          if re.search(r'examples.*\.py$', ap.strpath)])
@@ -287,19 +251,41 @@ def test_collect_gallery_files(sphinx_app_wrapper, tmpdir):
 
     tutorials_path = tmpdir.join('tutorials')
     dirs = [examples_path.strpath, tutorials_path.strpath]
-    collected_files = set(collect_gallery_files(dirs))
+    collected_files = set(collect_gallery_files(dirs, gallery_conf))
     expected_files = set(
         [ap.strpath for ap in abs_paths if re.search(r'.*\.py$', ap.strpath)])
 
     assert collected_files == expected_files
 
 
+def test_collect_gallery_files_ignore_pattern(tmpdir, gallery_conf):
+    """Test that ignore pattern example files are not collected."""
+    rel_filepaths = ['examples/file1.py',
+                     'examples/folder1/fileone.py',
+                     'examples/folder1/file2.py',
+                     'examples/folder2/fileone.py']
+
+    abs_paths = [tmpdir.join(rp) for rp in rel_filepaths]
+    for ap in abs_paths:
+        ap.ensure()
+
+    gallery_conf['ignore_pattern'] = r'one'
+    examples_path = tmpdir.join('examples')
+    dirs = [examples_path.strpath]
+    collected_files = set(collect_gallery_files(dirs, gallery_conf))
+    expected_files = set(
+        [ap.strpath for ap in abs_paths
+         if re.search(r'one', os.path.basename(ap.strpath)) is None])
+
+    assert collected_files == expected_files
+
+
 @pytest.mark.conf_file(content="""
 sphinx_gallery_conf = {
     'backreferences_dir' : os.path.join('modules', 'gen'),
     'examples_dirs': 'src',
     'gallery_dirs': ['ex'],
-    'binder': {'url': 'http://test1.com', 'org': 'org',
+    'binder': {'binderhub_url': 'http://test1.com', 'org': 'org',
                'repo': 'repo', 'branch': 'branch',
                'notebooks_dir': 'ntbk_folder',
                'dependencies': 'requirements.txt'}
@@ -317,7 +303,7 @@ def test_binder_copy_files(sphinx_app_wrapper, tmpdir):
     for i_file in ['plot_1', 'plot_2', 'plot_3']:
         assert os.path.exists(os.path.join(
             sphinx_app.outdir, 'ntbk_folder', gallery_conf['gallery_dirs'][0],
-            i_file+'.ipynb'))
+            i_file + '.ipynb'))
 
 
 @pytest.mark.conf_file(content="""
@@ -360,3 +346,53 @@ def test_examples_not_expected_to_pass(sphinx_app_wrapper):
     with pytest.raises(ValueError) as excinfo:
         sphinx_app_wrapper.build_sphinx_app()
     assert "expected to fail, but not failing" in str(excinfo.value)
+
+
+@pytest.mark.conf_file(content="""
+sphinx_gallery_conf = {
+    'first_notebook_cell': 2,
+}""")
+def test_first_notebook_cell_config(sphinx_app_wrapper):
+    from sphinx_gallery.gen_gallery import parse_config
+    # First cell must be str
+    with pytest.raises(ValueError):
+        parse_config(sphinx_app_wrapper.create_sphinx_app())
+
+
+@pytest.mark.conf_file(content="""
+sphinx_gallery_conf = {
+    'last_notebook_cell': 2,
+}""")
+def test_last_notebook_cell_config(sphinx_app_wrapper):
+    from sphinx_gallery.gen_gallery import parse_config
+    # First cell must be str
+    with pytest.raises(ValueError):
+        parse_config(sphinx_app_wrapper.create_sphinx_app())
+
+
+@pytest.mark.conf_file(content="""
+sphinx_gallery_conf = {
+    'backreferences_dir': False,
+}""")
+def test_backreferences_dir_config(sphinx_app_wrapper):
+    """Tests 'backreferences_dir' type checking."""
+    from sphinx_gallery.gen_gallery import parse_config
+    with pytest.raises(ValueError,
+                       match="The 'backreferences_dir' parameter must be of"):
+        parse_config(sphinx_app_wrapper.create_sphinx_app())
+
+
+@pytest.mark.conf_file(content="""
+import pathlib
+
+sphinx_gallery_conf = {
+    'backreferences_dir': pathlib.Path('.'),
+}""")
+def test_backreferences_dir_pathlib_config(sphinx_app_wrapper):
+    """Tests pathlib.Path does not raise exception."""
+    from sphinx_gallery.gen_gallery import parse_config
+    parse_config(sphinx_app_wrapper.create_sphinx_app())
+
+
+def test_write_computation_times_noop():
+    write_computation_times(None, None, [[[0]]])
